from __future__ import annotations
import typing as T
from pathlib import Path

# whatever you might like to convert
SUFFIXES = (
    ".doc",
    ".docx",
    ".odt",
    ".pages",
    ".pub",
    ".rtf",
    ".wpd",
    ".wps",
    ".wri",
    ".xls",
    ".xlsx",
    ".ods",
    ".wks",
    ".ppt",
    ".pptx",
    ".odp",
)


def docfinder(
    path: Path, suffixes: tuple[str, ...] = None, exclude: str = None
) -> T.Iterator[Path]:

    path = Path(path).expanduser().resolve()

    if not suffixes:
        suffixes = SUFFIXES

    if path.is_file():
        for f in [path]:
            yield f
    elif path.is_dir():
        pass
    else:
        raise FileNotFoundError(f"{path} is not a path or file")

    if exclude:
        for f in path.iterdir():
            if (
                f.is_file()
                and f.suffix.lower() in suffixes
                and not f.with_suffix(exclude).is_file()
            ):
                yield f
    else:
        for f in path.iterdir():
            if f.is_file() and f.suffix in suffixes:
                yield f



from pathlib import Path
import subprocess
import shutil
import argparse

from . import docfinder

LOEXE = shutil.which("soffice")


def doc2pdf(filein: Path):
    if not LOEXE:
        raise EnvironmentError("LibreOffice not found")

    cmd = [LOEXE, "--convert-to", "pdf", "--outdir", str(filein.parent), str(filein)]

    subprocess.check_call(cmd, stderr=subprocess.DEVNULL)


if __name__ == "__main__":
    """
    Convert DOC, DOCX, ODT, RTF, etc. to PDF using LibreOffice
    It leaves the original files alone, putting the PDF in the same directory.

    NOTE: LibreOffice conversion is not thread-safe--it will
    silently fail if simultaneous processes are launched--sometimes.
    For loop, as the command line globbing doesn't work either,
    even just from Terminal!
    """
    p = argparse.ArgumentParser()
    p.add_argument("path", help="path to work in")
    p.add_argument("-s", "--suffix", help="file extensions to convert", nargs="+")
    p = p.parse_args()

    for f in docfinder(p.path, p.suffix, exclude=".pdf"):
        doc2pdf(f)



from pathlib import Path
import subprocess
import argparse

from . import docfinder


def doc2print(filein: Path, exe: str):
    if exe == "libreoffice":
        cmd = ["soffice", "-p", str(filein)]
    elif exe == "winword":
        cmd = ["winword.exe", "/q", "/x", "/mFilePrintDefault", "/t", str(filein)]
    elif exe == "powerpoint":
        cmd = ["powerpnt.exe", "/P", str(filein)]
    elif exe == "wordpad":
        cmd = ["write.exe", "/p", str(filein)]
    elif exe == "notepad++":
        cmd = ["notepad++", "-quickPrint", str(filein)]
    elif exe == "acroread":
        cmd = [
            "C:/Program Files (x86)/Adobe/Acrobat Reader DC/Reader/acrord32.exe",
            "/p",
            str(filein),
        ]
    else:
        raise ValueError(f"unknown program {exe}")

    subprocess.check_call(cmd)


if __name__ == "__main__":
    """
    prints DOC, DOCX, RTF, PPT, XLS, etc. using specified program.
    Due to limitations of programs, the DEFAULT PRINTER is used.
    This might not be the printer you expect, so do this offline / directly plugged to printer.
    """
    p = argparse.ArgumentParser()
    p.add_argument("path", help="path to work in")
    p.add_argument("-s", "--suffix", help="file extensions to print", nargs="+")
    p.add_argument(
        "-exe",
        help="printing program",
        choices=["libreoffice", "winword", "wordpad", "notepad++", "powerpoint", "acroread"],
        default="libreoffice",
    )
    p = p.parse_args()

    if p.exe == "wordpad":
        print(
            "\nNOTE: WordPad sometimes wastes an extra page printed since WordPad has trouble"
            " decoding contemporary Word documents. \nConsider LibreOffice instead.\n"
        )

    for f in docfinder(p.path, p.suffix):
        yesno = input(f"press Enter to print: {f}   or 's' to skip or 'q' to quit  ")
        if yesno == "q":
            break
        elif yesno == "s":
            continue

        doc2print(f, p.exe)
